@@ -14,6 +14,7 @@ DOIT_CONFIG = {
     'minversion': '0.24.0',
     'default_tasks': ['pyflakes', 'ut'],
 #    'backend': 'sqlite3',
+    'forget_disable_default': True,
     }
 
 CODE_FILES = glob.glob("doit/*.py")
@@ -29,7 +30,7 @@ def task_pyflakes():
     yield flaker.tasks('tests/*.py')
 
 def run_test(test):
-    return not bool(pytest.main(test))
+    return not bool(pytest.main([test]))
     #return not bool(pytest.main("-v " + test))
 def task_ut():
     """run unit-tests"""
@@ -59,6 +60,14 @@ def task_coverage():
 DOC_ROOT = 'doc/'
 DOC_BUILD_PATH = DOC_ROOT + '_build/html/'
 
+def task_rm_index():
+    """remove/clean copied index.html if source changed"""
+    # work around https://github.com/sphinx-doc/sphinx/issues/1649
+    return {
+        'actions': ['cd doc && make clean'],
+        'file_dep': ['doc/index.html'],
+    }
+
 def task_docs():
     doc_files = glob.glob('doc/*.rst')
     doc_files += ['README.rst', 'CONTRIBUTING.md',
@@ -66,7 +75,7 @@ def task_docs():
     yield docs.spell(doc_files, 'doc/dictionary.txt')
     sphinx_opts = "-A include_analytics=1 -A include_donate=1"
     yield docs.sphinx(DOC_ROOT, DOC_BUILD_PATH, sphinx_opts=sphinx_opts,
-                      task_dep=['spell'])
+                      task_dep=['spell', 'rm_index'])
 
 def task_samples_check():
     """check samples are at least runnuable without error"""
@@ -77,6 +86,7 @@ def task_samples_check():
         'taskresult.py',  # uses mercurial
         'tar.py',  # uses mercurial
         'calc_dep.py',  # uses files not created by the script
+        'report_deps.py',  # uses files not created by the script
         'doit_config.py',  # no tasks defined
     ]
     exclude = set('doc/samples/{}'.format(m) for m in black_list)
@@ -123,8 +133,36 @@ def task_package():
     yield pkg.revision_git()
     yield pkg.manifest_git()
     yield pkg.sdist()
-    yield pkg.sdist_upload()
+    # yield pkg.sdist_upload()
 
 
 
+def task_sample():
+    def show_meta(task):
+        print(f'META: {task.meta}')
+    return {
+        'actions': ['echo %(param1)s', show_meta],
+        'params': [
+            {
+                'name':'param1',
+                'short':'p',
+                'default':'default value',
+            },
+        ],
+        'meta': {'tags': ['sample']},
+        'verbosity': 2,
+    }
+
+
 # doit -f ../doit-recipes/deps/deps.py -d . --reporter=executed-only
+
+
+from doit import task_params
+
+@task_params([{"name": "howmany", "default": 3, "type": int, "long": "howmany"}])
+def task_subtasks(howmany):
+    for i in range(howmany):
+        yield {
+            "name": i,
+            "actions": [f"echo I can count to {howmany}: {i}"],
+        }
