@@ -1,5 +1,6 @@
 import os
 import inspect
+from operator import attrgetter
 
 import pytest
 
@@ -7,7 +8,7 @@ from doit.exceptions import InvalidDodoFile, InvalidCommand
 from doit.task import InvalidTask, DelayedLoader, Task
 from doit.loader import flat_generator, get_module
 from doit.loader import load_tasks, load_doit_config, generate_tasks
-from doit.loader import create_after
+from doit.loader import create_after, task_params
 
 
 class TestFlatGenerator(object):
@@ -68,6 +69,12 @@ class TestLoadTasks(object):
         def task_yyy2():
             return {'actions':None}
 
+        def task_meta():
+            return {
+                'actions' : ['do nothing'],
+                'meta'    : { 'a' : ['b', 'c']},
+            }
+
         def bad_seed(): pass
         task_nono = 5
         task_nono # pyflakes
@@ -75,9 +82,10 @@ class TestLoadTasks(object):
 
     def testNormalCase(self, dodo):
         task_list = load_tasks(dodo)
-        assert 2 == len(task_list)
+        assert 3 == len(task_list)
         assert 'xxx1' == task_list[0].name
         assert 'yyy2' == task_list[1].name
+        assert 'meta' == task_list[2].name
 
     def testCreateAfterDecorator(self):
         @create_after('yyy2')
@@ -128,7 +136,47 @@ class TestLoadTasks(object):
         f_task = tasks['foo']
         assert f_task.loader.task_dep == 'yyy2'
         assert f_task.loader.creator == task_zzz3
-        assert tasks['bar'].loader is tasks['foo'].loader
+        assert tasks['bar'].loader.task_dep == tasks['foo'].loader.task_dep
+        assert tasks['foo'].doc == 'not loaded task doc'
+
+        # make sure doit can be executed more then once in single process GH#381
+        list2 = load_tasks(dodo, allow_delayed=True)
+        tasks2 = {t.name:t for t in list2}
+        assert tasks['bar'].loader is not tasks2['bar'].loader
+
+    def testCreateAfterDecoratorOnMethod(self):
+        'Check that class-defined tasks are loaded as bound methods'
+        class Tasks:
+            @create_after('yyy2')
+            def task_zzz3(): # pragma: no cover
+                pass
+
+        # create_after annotates the function
+        task_list = load_tasks({'task_zzz3': Tasks().task_zzz3}, allow_delayed=True)
+        tasks = {t.name:t for t in task_list}
+        task_zzz3 = tasks['zzz3']
+        assert isinstance(task_zzz3.loader, DelayedLoader)
+        # check creator is a bound method, not a plain function
+        assert getattr(task_zzz3.loader.creator, '__self__', None) is not None
+
+    def testCreateAfterDecoratorOnMethodWithParams(self, dodo):
+        'Check that class-defined tasks support the creates argument of @create_after'
+        class Tasks:
+            @create_after('yyy2', creates=['foo', 'bar'])
+            def task_zzz3(): # pragma: no cover
+                '''not loaded task doc'''
+                raise Exception('Cant be executed on load phase')
+
+        # placeholder task is created with `loader` attribute
+        task_list = load_tasks({'task_zzz3': Tasks().task_zzz3}, allow_delayed=True)
+        tasks = {t.name:t for t in task_list}
+        assert 'zzz3' not in tasks
+        f_task = tasks['foo']
+        assert f_task.loader.task_dep == 'yyy2'
+        assert getattr(f_task.loader.creator, '__self__', None) is not None
+        # loaders are not the same because of #381 (multiple execution on same process)
+        # But this is not a problem because once the task already exists, the loader is just not used.
+        # assert tasks['bar'].loader is tasks['foo'].loader
         assert tasks['foo'].doc == 'not loaded task doc'
 
     def testNameInBlacklist(self):
@@ -139,6 +187,10 @@ class TestLoadTasks(object):
         task_list = load_tasks(dodo)
         assert "task doc" == task_list[0].doc
 
+    def testMetaInfo(self, dodo):
+        task_list = load_tasks(dodo)
+        assert task_list[2].meta == {'a': ['b', 'c']}
+
     def testUse_create_doit_tasks(self):
         def original(): pass
         def creator():
@@ -175,6 +227,137 @@ class TestLoadTasks(object):
         assert 'method2' == task_list[1].name
 
 
+
+class TestTaskGeneratorParams(object):
+
+    def test_task_params_annotations(self):
+        params = [{"name": "foo", "default": "bar", "long": "foo"}]
+        func = task_params(params)(lambda: 1)
+        assert func._task_creator_params == params
+
+    def test_default(self):
+        'Ensure that a task parameter can be passed to the task generator.'
+
+        @task_params([{"name": "foo", "default": "bar", "long": "foo"}])
+        def task_foo(foo):
+            return {
+                'actions': [],
+                'doc': foo
+            }
+        task_list = load_tasks({'task_foo': task_foo})
+        task = task_list.pop()
+        assert task.doc == 'bar'
+        task.init_options()
+        assert task.options['foo'] == 'bar'
+
+    def test_args(self):
+        'Ensure that a task generator parameter can be set from the command line.'
+        @task_params([{"name": "fp", "default": "default p", "long": "fp"}])
+        def task_foo(fp):
+            return {
+                'actions': [],
+                'doc': fp
+            }
+        args = ['foo', '--fp=from_arg']
+        task_list = load_tasks({'task_foo': task_foo}, args=args)
+        task = task_list.pop()
+        assert task.doc == 'from_arg'
+
+
+    def test_args_second(self):
+        def task_bar():
+            return {'actions': []}
+
+        @task_params([{"name": "foo", "default": "placeholder", "long": "foo"}])
+        def task_foo(foo):
+            return {
+                'actions': [],
+                'doc': foo
+            }
+        args = ['bar', 'foo', '--foo=from_arg']
+        task_list = load_tasks({'task_foo': task_foo, 'task_bar': task_bar}, args=args)
+        assert len(task_list) == 2
+        bar, foo = sorted(task_list, key=attrgetter('name'))
+        assert foo.name == 'foo'
+        assert foo.doc == 'from_arg'
+
+
+    def test_config(self):
+        @task_params([{"name": "fp", "default": "default p", "long": "fp"}])
+        def task_foo(fp):
+            return {
+                'actions': [],
+                'doc': fp
+            }
+        config = {'task:foo': {'fp': 'from_config'}}
+        task_list = load_tasks({'task_foo': task_foo}, args=(), config=config)
+        task = task_list.pop()
+        assert task.doc == 'from_config'
+        # config is overwritten from args
+        args = ['foo', '--fp=from_arg']
+        task_list2 = load_tasks({'task_foo': task_foo}, args=args, config=config)
+        task2 = task_list2.pop()
+        assert task2.doc == 'from_arg'
+
+
+    def test_method(self):
+        'Ensure that a task parameter can be passed to the task generator defined as a class method.'
+        class Tasks(object):
+            @task_params([{"name": "param1", "default": "placeholder", "long": "param1"}])
+            def task_foo(self, param1):
+                for i in range(2):
+                    yield {
+                        'name': 'subtask' + str(i),
+                        'actions': [],
+                        'doc': param1,
+                    }
+
+        foo = Tasks().task_foo
+        task_list = load_tasks({'task_foo': foo}, args=('foo', '--param1=my_val'))
+
+        assert len(task_list) == 3
+        tasks = {t.name: t for t in task_list}
+
+        assert len(tasks['foo'].params) == 0
+        assert len(tasks['foo'].creator_params) == 1
+        assert tasks['foo'].doc == ''
+
+        assert len(tasks['foo:subtask0'].params) == 0
+        assert tasks['foo:subtask0'].doc == 'my_val'
+
+
+    def test_delayed(self):
+        @create_after()
+        @task_params([{"name": "fp", "default": "default p", "long": "fp"}])
+        def task_foo(fp):
+            return {
+                'actions': [],
+                'doc': fp
+            }
+        args = ['foo', '--fp=from_arg']
+        task_list = load_tasks({'task_foo': task_foo}, allow_delayed=True, args=args)
+        task = task_list.pop()
+        assert task.name == 'foo'
+        assert task.loader.kwargs == {'fp': 'from_arg'}
+        assert len(task.creator_params) == 1
+
+
+    def test_dup_param(self):
+        'Ensure that `params` field and @task_params definitions are prohibited'
+        @task_params([{"name": "foo", "default": "decorator", "long": "foo"}])
+        def task_dup(foo):
+            return {
+                'actions': [],
+                'params': [{"name": "bar", "default": "dict", "long": "bar"}],
+            }
+
+        with pytest.raises(InvalidTask) as exc_info:
+            load_tasks({'task_dup': task_dup})
+        assert ('attribute can not be used in conjuction with' in str(exc_info.value))
+
+
+
+
 class TestDodoConfig(object):
 
     def testConfigType_Error(self):
